@@ -48,13 +48,11 @@
 
 #endif  // FORCE_PR_LOG
 
-#if MOZILLA_MAJOR_VERSION > 9
-
 #define IPCBool bool
 
-#else
+#if MOZILLA_MAJOR_VERSION > 17
 
-#define IPCBool PRBool
+#define nsCAutoString nsAutoCString
 
 #endif // MOZILLA_MAJOR_VERSION
 
