@@ -49,7 +49,9 @@
 #include "nsIMsgMailSession.h"
 #include "nsIMimeMiscStatus.h"
 #include "nsIEnigMimeHeaderSink.h"
+#include "nsIAsyncInputStream.h"
 #include "nsIThread.h"
+#include "nsIEventTarget.h"
 #include "nsEnigMimeDecrypt.h"
 #include "nsIPipeTransport.h"
 #include "nsIIPCBuffer.h"
@@ -69,28 +71,34 @@ static const PRUint32 kCharMax = 1024;
 // nsEnigMimeDecrypt implementation
 
 // nsISupports implementation
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsEnigMimeDecrypt,
-                              nsIEnigMimeDecrypt)
+NS_IMPL_THREADSAFE_ISUPPORTS2(nsEnigMimeDecrypt,
+                              nsIEnigMimeDecrypt,
+                              nsIPipeReader)
 
 // nsEnigMimeDecrypt implementation
 nsEnigMimeDecrypt::nsEnigMimeDecrypt()
   : mInitialized(PR_FALSE),
     mVerifyOnly(PR_FALSE),
     mRfc2015(PR_FALSE),
+    mDone(PR_FALSE),
 
     mInputLen(0),
     mOutputLen(0),
+    mIterations(0),
+    mCtFound(-1),
 
-    mBuffer(nsnull),
-    mListener(nsnull),
-    mPipeTrans(nsnull)
+    mBuffer(NULL),
+    mListener(NULL),
+    mPipeTrans(NULL),
+    mSecurityInfo(NULL),
+    mUri(NULL)
 {
   nsresult rv;
 
   NS_INIT_ISUPPORTS();
 
 #ifdef PR_LOGGING
-  if (gEnigMimeDecryptLog == nsnull) {
+  if (gEnigMimeDecryptLog == NULL) {
     gEnigMimeDecryptLog = PR_NewLogModule("nsEnigMimeDecrypt");
   }
 #endif
@@ -152,7 +160,7 @@ nsEnigMimeDecrypt::Init(EMBool verifyOnly,
     if (NS_FAILED(rv)) return rv;
 
     rv = mListener->Init((nsIStreamListener*)(mBuffer),
-                         nsnull, "", "", 1, PR_FALSE, PR_TRUE, nsnull);
+                         NULL, "", "", 1, PR_FALSE, PR_TRUE, NULL);
     if (NS_FAILED(rv)) return rv;
   }
 
@@ -172,16 +180,16 @@ nsEnigMimeDecrypt::Finalize()
 
   if (mPipeTrans) {
     mPipeTrans->Terminate();
-    mPipeTrans = nsnull;
+    mPipeTrans = NULL;
   }
 
   if (mListener) {
-    mListener = nsnull;
+    mListener = NULL;
   }
 
   if (mBuffer) {
     mBuffer->Shutdown();
-    mBuffer = nsnull;
+    mBuffer = NULL;
   }
 
   return NS_OK;
@@ -195,7 +203,7 @@ nsEnigMimeDecrypt::Write(const char *buf, PRUint32 buf_size)
     return NS_ERROR_NOT_INITIALIZED;
 
   if (mListener)
-    mListener->Write(buf, buf_size, nsnull, nsnull);
+    mListener->Write(buf, buf_size, NULL, NULL);
   else
     mBuffer->WriteBuf(buf, buf_size);
 
@@ -234,10 +242,11 @@ nsEnigMimeDecrypt::FinishAux(nsIMsgWindow* msgWindow, nsIURI* uri)
   nsCOMPtr<nsIThread> myThread;
   rv = ENIG_GET_THREAD(myThread);
 
+  mUri = uri;
   nsCAutoString uriSpec("");
 
   if (mListener) {
-    rv = mListener->OnStopRequest(nsnull, nsnull, 0);
+    rv = mListener->OnStopRequest(NULL, NULL, NS_OK);
     if (NS_FAILED(rv))
       return rv;
 
@@ -250,21 +259,20 @@ nsEnigMimeDecrypt::FinishAux(nsIMsgWindow* msgWindow, nsIURI* uri)
       return NS_ERROR_FAILURE;
     }
 
-    mListener = nsnull;
+    mListener = NULL;
   }
 
-  rv = mBuffer->OnStopRequest(nsnull, nsnull, 0);
+  rv = mBuffer->OnStopRequest(NULL, NULL, NS_OK);
   if (NS_FAILED(rv))
     return rv;
 
-  nsCOMPtr<nsISupports> securityInfo;
   if (msgWindow) {
     nsCOMPtr<nsIMsgHeaderSink> headerSink;
     msgWindow->GetMsgHeaderSink(getter_AddRefs(headerSink));
     if (headerSink)
-        headerSink->GetSecurityInfo(getter_AddRefs(securityInfo));
+        headerSink->GetSecurityInfo(getter_AddRefs(mSecurityInfo));
   }
-  DEBUG_LOG(("nsEnigMimeDecrypt::FinishAux: securityInfo=%p\n", securityInfo.get()));
+  DEBUG_LOG(("nsEnigMimeDecrypt::FinishAux: securityInfo=%p\n", mSecurityInfo.get()));
 
   nsCOMPtr<nsIPrompt> prompter;
   if (msgWindow) {
@@ -291,111 +299,154 @@ nsEnigMimeDecrypt::FinishAux(nsIMsgWindow* msgWindow, nsIURI* uri)
   EMBool noOutput = PR_FALSE;
   PRUint32 statusFlags;
 
-  rv = enigmailSvc->DecryptMessageStart(nsnull,
+  rv = enigmailSvc->DecryptMessageStart(NULL,
                                         prompter,
                                         mVerifyOnly,
                                         noOutput,
-                                        nsnull,
+                                        NULL,
                                         &statusFlags,
                                         getter_Copies(errorMsg),
                                         getter_AddRefs(mPipeTrans) );
   if (NS_FAILED(rv)) return rv;
 
   if (!mPipeTrans) {
-    if (securityInfo) {
-      nsCOMPtr<nsIEnigMimeHeaderSink> enigHeaderSink = do_QueryInterface(securityInfo);
+    if (mSecurityInfo) {
+      nsCOMPtr<nsIEnigMimeHeaderSink> enigHeaderSink = do_QueryInterface(mSecurityInfo);
       if (enigHeaderSink) {
         NS_NAMED_LITERAL_STRING(nullString, "");
-        rv = enigHeaderSink->UpdateSecurityStatus(uriSpec, -1, statusFlags, nullString.get(), nullString.get(), nullString.get(), errorMsg.get(), nullString.get());
+        rv = enigHeaderSink->UpdateSecurityStatus(uriSpec, -1, statusFlags, nullString.get(), nullString.get(), nullString.get(), errorMsg.get(), nullString.get(), mUri);
       }
     }
 
     return NS_ERROR_FAILURE;
   }
 
-  nsCOMPtr<nsIInputStream> plainStream;
-  rv = mPipeTrans->OpenInputStream(0, PRUint32(-1), 0,
-                                  getter_AddRefs(plainStream));
-  if (NS_FAILED(rv)) return rv;
+  mIterations = 0;
+  mCtFound = -1;
+  nsCOMPtr<nsIInputStream> plainStream = NULL;
+
+  // read via pipeTransport.jsm
+  nsCOMPtr<nsIRequest> request;
+  rv = mPipeTrans->ReadInputStream(this, getter_AddRefs(request));
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Write buffered data asyncronously to process
   nsCOMPtr<nsIInputStream> bufStream;
   rv = mBuffer->OpenInputStream(getter_AddRefs(bufStream));
   if (NS_FAILED(rv)) return rv;
 
+#if MOZILLA_MAJOR_VERSION < 17
   PRUint32 available;
+#else
+  PRUint64 available;
+#endif
+
   rv = bufStream->Available(&available);
   if (NS_FAILED(rv)) return rv;
 
   DEBUG_LOG(("nsEnigMimeDecrypt::FinishAux: available=%d\n", available));
 
-  rv = mPipeTrans->WriteAsync(bufStream, available, PR_TRUE);
+  rv = mPipeTrans->WriteAsync(bufStream, (PRUint32) available, PR_TRUE);
   if (NS_FAILED(rv)) return rv;
 
-  PRUint32 readCount, iterations;
-  int ctFound = -1;
-  char buf[kCharMax];
-  iterations = 0;
-  int status;
-  while (1) {
-    ++iterations;
-    // Read synchronously
+  // read via pipeTransport.jsm
 
-    rv = plainStream->Read((char *) buf, kCharMax, &readCount);
-    if (NS_FAILED(rv)) return rv;
+  nsCOMPtr<nsIThread> currentThread;
+  rv = ENIG_GET_THREAD(currentThread);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    if (!readCount) break;
+  mDone = PR_FALSE;
 
-    if (iterations==1 && readCount > 25) {
-      // add mime boundaries around text/plain message (bug 6627)
-      if (PL_strncasecmp("content-type:", buf, 13)==0) {
-        PRUint32 whitespace=13;
-        while((whitespace<readCount) && buf[whitespace] &&
-              ((buf[whitespace]==' ') || (buf[whitespace]=='\t'))) { whitespace++; }
-        if (buf[whitespace] && (whitespace<readCount)) {
-          ctFound=PL_strncasecmp(buf + whitespace, "text/plain", 10);
-          if (ctFound != 0) {
-            ctFound=PL_strncasecmp(buf + whitespace, "text/html", 9);
-          }
-        }
-        if (ctFound==0) {
-          char* header = PR_smprintf(
-          "Content-Type: multipart/mixed; boundary=\"enigDummy\""
-          "\n\n--enigDummy\n");
-          PR_SetError(0,0);
-          status = mOutputFun(header, strlen(header), mOutputClosure);
-          if (status < 0) {
-            PR_SetError(status, 0);
-            mOutputFun = NULL;
-            mOutputClosure = NULL;
-
-            return NS_ERROR_FAILURE;
-          }
-          mOutputLen += strlen(header);
-        }
-      }
-    }
-
-    if (readCount < kCharMax) {
-      // make sure we can continue to write later
-      if (buf[readCount-1]==0) --readCount;
-    }
-
-    PR_SetError(0,0);
-    status = mOutputFun(buf, readCount, mOutputClosure);
-    if (status < 0) {
-      PR_SetError(status, 0);
-      mOutputFun = NULL;
-      mOutputClosure = NULL;
-
-      return NS_ERROR_FAILURE;
-    }
-
-    mOutputLen += readCount;
+  // wait with returning until message is completely processed
+  // (simulate synchronous function)
+  while (! mDone) {
+    EMBool pendingEvents;
+    rv = currentThread->ProcessNextEvent(PR_TRUE, &pendingEvents);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
+  return NS_OK;
+}
 
-  if (ctFound==0) {
+nsresult
+nsEnigMimeDecrypt::ProcessPlainData(char* buf, PRUint32 readCount)
+{
+  DEBUG_LOG(("nsEnigMimeDecrypt::ProcessPlainData: readCount=%d\n", readCount));
+
+  int status;
+  ++mIterations;
+  // Read synchronously
+
+
+  if (mIterations == 1 && readCount > 25) {
+    // add mime boundaries around text/plain message (bug 6627)
+    if (PL_strncasecmp("content-type:", buf, 13)==0) {
+      PRUint32 whitespace=13;
+      while((whitespace<readCount) && buf[whitespace] &&
+            ((buf[whitespace]==' ') || (buf[whitespace]=='\t'))) { whitespace++; }
+      if (buf[whitespace] && (whitespace<readCount)) {
+        mCtFound = PL_strncasecmp(buf + whitespace, "text/plain", 10);
+        if (mCtFound != 0) {
+          mCtFound=PL_strncasecmp(buf + whitespace, "text/html", 9);
+        }
+      }
+      if (mCtFound == 0) {
+        char* header = PR_smprintf(
+        "Content-Type: multipart/mixed; boundary=\"enigDummy\""
+        "\n\n--enigDummy\n");
+        PR_SetError(0,0);
+        status = mOutputFun(header, strlen(header), mOutputClosure);
+        if (status < 0) {
+          PR_SetError(status, 0);
+          mOutputFun = NULL;
+          mOutputClosure = NULL;
+
+          return NS_ERROR_FAILURE;
+        }
+        mOutputLen += strlen(header);
+      }
+    }
+  }
+
+  if (readCount < kCharMax) {
+    // make sure we can continue to write later
+    if (buf[readCount-1]==0) --readCount;
+  }
+
+  PR_SetError(0,0);
+  status = mOutputFun(buf, readCount, mOutputClosure);
+  if (status < 0) {
+    PR_SetError(status, 0);
+    mOutputFun = NULL;
+    mOutputClosure = NULL;
+
+    return NS_ERROR_FAILURE;
+  }
+
+  mOutputLen += readCount;
+
+  return NS_OK;
+} // loop end
+
+
+nsresult
+nsEnigMimeDecrypt::ProcessEnd(nsIInputStream* plainStream)
+{
+  DEBUG_LOG(("nsEnigMimeDecrypt::ProcessEnd:\n"));
+
+  char buf[kCharMax];
+  nsresult rv;
+  nsString errorMsg;
+  nsCAutoString uriSpec("");
+
+
+  nsCOMPtr<nsIEnigmail> enigmailSvc = do_GetService(NS_ENIGMAIL_CONTRACTID, &rv);
+  if (NS_FAILED(rv))
+    return rv;
+
+DEBUG_LOG(("nsEnigMimeDecrypt::ProcessEnd: got Enigmail Svc\n"));
+
+  if (mCtFound==0) {
     // add mime boundaries around text/plain message (bug 6627)
     PR_SetError(0,0);
     strcpy(buf, "\n\n--enigDummy--\n");
@@ -405,6 +456,7 @@ nsEnigMimeDecrypt::FinishAux(nsIMsgWindow* msgWindow, nsIURI* uri)
       PR_SetError(status, 0);
       mOutputFun = NULL;
       mOutputClosure = NULL;
+DEBUG_LOG(("nsEnigMimeDecrypt::ProcessEnd: error 1\n"));
 
       return NS_ERROR_FAILURE;
     }
@@ -424,7 +476,7 @@ nsEnigMimeDecrypt::FinishAux(nsIMsgWindow* msgWindow, nsIURI* uri)
   PR_SetError(0,0);
 
   // Close input stream
-  plainStream->Close();
+  if (plainStream) plainStream->Close();
 
   // Close buffer
   mBuffer->Shutdown();
@@ -434,8 +486,10 @@ nsEnigMimeDecrypt::FinishAux(nsIMsgWindow* msgWindow, nsIURI* uri)
   nsString userId;
   nsString sigDate;
   nsString blockSeparation;
+  PRUint32 statusFlags;
 
   PRUint32 uiFlags = nsIEnigmail::UI_PGP_MIME;
+  EMBool noOutput = PR_FALSE;
 
   rv = enigmailSvc->DecryptMessageEnd(uiFlags,
                                       mOutputLen,
@@ -449,19 +503,57 @@ nsEnigMimeDecrypt::FinishAux(nsIMsgWindow* msgWindow, nsIURI* uri)
                                       getter_Copies(errorMsg),
                                       getter_Copies(blockSeparation),
                                       &exitCode);
+
   if (NS_FAILED(rv)) return rv;
 
-  if (securityInfo) {
-    nsCOMPtr<nsIEnigMimeHeaderSink> enigHeaderSink = do_QueryInterface(securityInfo);
+  if (mSecurityInfo) {
+    nsCOMPtr<nsIEnigMimeHeaderSink> enigHeaderSink = do_QueryInterface(mSecurityInfo);
     if (enigHeaderSink) {
-      rv = enigHeaderSink->UpdateSecurityStatus(uriSpec, exitCode, statusFlags, keyId.get(), userId.get(), sigDate.get(), errorMsg.get(), blockSeparation.get());
+      rv = enigHeaderSink->UpdateSecurityStatus(uriSpec, exitCode, statusFlags, keyId.get(), userId.get(), sigDate.get(), errorMsg.get(), blockSeparation.get(), mUri);
     }
   }
 
   if (exitCode != 0) {
-    DEBUG_LOG(("nsEnigMimeDecrypt::FinishAux: ERROR EXIT %d\n", exitCode));
     return NS_ERROR_FAILURE;
   }
 
   return NS_OK;
 }
+
+
+NS_IMETHODIMP
+nsEnigMimeDecrypt::ReadData(const char* buf,
+                            PRUint32 count)
+{
+  nsresult rv;
+  DEBUG_LOG(("nsEnigMimeDecrypt::ReadData: count=%d\n", count));
+
+  NS_ENSURE_ARG(buf);
+
+/*
+  PRUint32 readCount;
+  char buf[count + 1];
+
+  rv = stream->Read((char*) buf, count, &readCount);
+  NS_ENSURE_SUCCESS(rv, rv);
+*/
+  if (count) {
+    rv = ProcessPlainData((char *) buf, count);
+    return rv;
+  }
+
+  return NS_OK;
+
+}
+
+
+NS_IMETHODIMP
+nsEnigMimeDecrypt::StopRequest(PRUint32 status)
+{
+  DEBUG_LOG(("nsEnigMimeDecrypt::StopRequest:\n"));
+
+  ProcessEnd(NULL);
+  mDone = PR_TRUE;
+
+  return NS_OK;
+}
