@@ -94,7 +94,7 @@ nsEnigMimeListener::nsEnigMimeListener()
     mContentLength(-1),
 
     mDecodeContent(PR_FALSE),
-    mDecoderData(nsnull),
+    mDecoderData(NULL),
 
     mLinebreak(""),
     mHeaders(""),
@@ -107,18 +107,18 @@ nsEnigMimeListener::nsEnigMimeListener()
     mMaxHeaderBytes(0),
     mDataOffset(0),
 
-    mStreamBuf(nsnull),
+    mStreamBuf(NULL),
     mStreamOffset(0),
     mStreamLength(0),
     mSubPartTreatment(PR_FALSE),
 
-    mListener(nsnull),
-    mContext(nsnull)
+    mListener(NULL),
+    mContext(NULL)
 {
     NS_INIT_ISUPPORTS();
 
 #ifdef PR_LOGGING
-  if (gEnigMimeListenerLog == nsnull) {
+  if (gEnigMimeListenerLog == NULL) {
     gEnigMimeListenerLog = PR_NewLogModule("nsEnigMimeListener");
   }
 #endif
@@ -146,12 +146,12 @@ nsEnigMimeListener::~nsEnigMimeListener()
   if (mDecoderData) {
     // Clear decoder buffer
     MimeDecoderDestroy(mDecoderData, PR_FALSE);
-    mDecoderData = nsnull;
+    mDecoderData = NULL;
   }
 
   // Release owning refs
-  mListener = nsnull;
-  mContext = nsnull;
+  mListener = NULL;
+  mContext = NULL;
 }
 
 
@@ -161,8 +161,8 @@ nsEnigMimeListener::~nsEnigMimeListener()
 
 NS_IMETHODIMP
 nsEnigMimeListener::Init(nsIStreamListener* listener, nsISupports* ctxt,
-                         PRUint32 maxHeaderBytes, PRBool skipHeaders,
-                         PRBool skipBody, PRBool decodeContent)
+                         PRUint32 maxHeaderBytes, EMBool skipHeaders,
+                         EMBool skipBody, EMBool decodeContent)
 {
   DEBUG_LOG(("nsEnigMimeListener::Init: (%p) %d, %d, %d, %d\n", this,
              maxHeaderBytes, skipHeaders, skipBody, decodeContent));
@@ -201,7 +201,7 @@ nsEnigMimeListener::Write(const char* buf, PRUint32 count,
     return Transmit(buf, count, aRequest, aContext);
 
   // Search for headers
-  PRBool startingRequest = HeaderSearch(buf, count);
+  EMBool startingRequest = HeaderSearch(buf, count);
   if (!startingRequest)
     return NS_OK;
 
@@ -222,7 +222,7 @@ EnigMimeListener_write(const char *buf, PRInt32 size, void *closure)
 
   nsEnigMimeListener* enigMimeListener = (nsEnigMimeListener *) closure;
 
-  return enigMimeListener->SendStream(buf, size, nsnull, nsnull);
+  return enigMimeListener->SendStream(buf, size, NULL, NULL);
 }
 
 
@@ -399,7 +399,7 @@ nsEnigMimeListener::OnStopRequest(nsIRequest* aRequest,
   if (mDecoderData) {
     // Clear decoder buffer
     MimeDecoderDestroy(mDecoderData, PR_FALSE);
-    mDecoderData = nsnull;
+    mDecoderData = NULL;
   }
 
   if (mListener) {
@@ -411,8 +411,8 @@ nsEnigMimeListener::OnStopRequest(nsIRequest* aRequest,
   }
 
   // Release owning refs
-  mListener = nsnull;
-  mContext = nsnull;
+  mListener = NULL;
+  mContext = NULL;
 
   return (aStatus == NS_BINDING_ABORTED) ? NS_ERROR_FAILURE : NS_OK;
 }
@@ -425,7 +425,11 @@ NS_IMETHODIMP
 nsEnigMimeListener::OnDataAvailable(nsIRequest* aRequest,
                                     nsISupports* aContext,
                                     nsIInputStream *aInputStream,
+#if MOZILLA_MAJOR_VERSION < 18
                                     PRUint32 aSourceOffset,
+#else
+                                    PRUint64 aSourceOffset,
+#endif
                                     PRUint32 aLength)
 {
   nsresult rv = NS_OK;
@@ -517,7 +521,7 @@ nsEnigMimeListener::StartRequest(nsIRequest* aRequest, nsISupports* aContext)
 }
 
 
-PRBool
+EMBool
 nsEnigMimeListener::HeaderSearch(const char* buf, PRUint32 count)
 {
   DEBUG_LOG(("nsEnigMimeListener::HeaderSearch: (%p) count=%d\n", this, count));
@@ -535,11 +539,11 @@ nsEnigMimeListener::HeaderSearch(const char* buf, PRUint32 count)
   PRUint32 bytesAvailable = mMaxHeaderBytes - mDataStr.Length();
   NS_ASSERTION(bytesAvailable > 0, "bytesAvailable <= 0");
 
-  PRBool lastSegment = (bytesAvailable <= count);
+  EMBool lastSegment = (bytesAvailable <= count);
 
   PRUint32 scanLen = lastSegment ? bytesAvailable : count;
 
-  PRBool headersFound = PR_FALSE;
+  EMBool headersFound = PR_FALSE;
   PRUint32 offset = 0;
   PRUint32 startOffset = 0;
   PRUint32 j = 0;
@@ -877,7 +881,11 @@ nsEnigMimeListener::ParseHeader(const char* header, PRUint32 count)
 ///////////////////////////////////////////////////////////////////////////////
 
 NS_IMETHODIMP
+#if MOZILLA_MAJOR_VERSION < 17
 nsEnigMimeListener::Available(PRUint32* _retval)
+#else
+nsEnigMimeListener::Available(PRUint64* _retval)
+#endif
 {
   if (!_retval)
     return NS_ERROR_NULL_POINTER;
@@ -952,7 +960,7 @@ nsEnigMimeListener::ReadSegments(nsWriteSegmentFun writer,
 }
 
 NS_IMETHODIMP
-nsEnigMimeListener::IsNonBlocking(PRBool *aNonBlocking)
+nsEnigMimeListener::IsNonBlocking(EMBool *aNonBlocking)
 {
   DEBUG_LOG(("nsEnigMimeListener::IsNonBlocking: \n"));
 
@@ -961,14 +969,14 @@ nsEnigMimeListener::IsNonBlocking(PRBool *aNonBlocking)
 }
 
 NS_IMETHODIMP
-nsEnigMimeListener::GetSubPartTreatment(PRBool* aSubPartTreatment)
+nsEnigMimeListener::GetSubPartTreatment(EMBool* aSubPartTreatment)
 {
   *aSubPartTreatment = mSubPartTreatment;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsEnigMimeListener::SetSubPartTreatment(PRBool aSubPartTreatment)
+nsEnigMimeListener::SetSubPartTreatment(EMBool aSubPartTreatment)
 {
   DEBUG_LOG(("nsEnigMimeListener::SetSubPartTreatment: %d\n", aSubPartTreatment));
 
@@ -980,7 +988,7 @@ NS_IMETHODIMP
 nsEnigMimeListener::Close()
 {
   DEBUG_LOG(("nsEnigMimeListener::Close: (%p)\n", this));
-  mStreamBuf = nsnull;
+  mStreamBuf = NULL;
   mStreamOffset = 0;
   mStreamLength = 0;
   return NS_OK;
