@@ -35,6 +35,7 @@
 
 Components.utils.import("resource://enigmail/enigmailCommon.jsm");
 Components.utils.import("resource://enigmail/commonFuncs.jsm");
+Components.utils.import("resource://enigmail/mimeDecrypt.jsm");
 
 if (! Enigmail) var Enigmail = {};
 
@@ -412,11 +413,7 @@ Enigmail.hdrView = {
       }
 
       if (statusFlags & nsIEnigmail.DECRYPTION_OKAY) {
-        var enigMimeService = Components.classes[EnigmailCommon.ENIGMIMESERVICE_CONTRACTID].getService(Components.interfaces.nsIEnigMimeService);
-        if (enigMimeService)
-        {
-          enigMimeService.rememberEncrypted(this.lastEncryptedMsgKey);
-        }
+        EnigmailCommon.rememberEncryptedUri(this.lastEncryptedMsgKey);
 
         // Display encrypted icon
         gEncryptedUINode.setAttribute("encrypted", "ok");
@@ -538,6 +535,11 @@ Enigmail.hdrView = {
         try {
 
           Enigmail.hdrView.statusBarHide();
+          if (gFolderDisplay.selectedMessages.length != 1) {
+            EnigmailDecrypt.disableDecrypt();
+          }
+          else
+            EnigmailDecrypt.setMsgWindow(msgWindow, Enigmail.msg.getCurrentMsgUriSpec());
 
           var statusText = document.getElementById("enigmailStatusText");
           if (statusText) statusText.value="";
@@ -635,12 +637,8 @@ Enigmail.hdrView = {
   {
     if (Enigmail.hdrView.lastEncryptedMsgKey)
     {
-      var enigMimeService = Components.classes[EnigmailCommon.ENIGMIMESERVICE_CONTRACTID].
-                              getService(Components.interfaces.nsIEnigMimeService);
-      if (enigMimeService) {
-        enigMimeService.forgetEncrypted(Enigmail.hdrView.lastEncryptedMsgKey);
-        Enigmail.hdrView.lastEncryptedMsgKey = null;
-      }
+      EnigmailCommon.forgetEncryptedUri(Enigmail.hdrView.lastEncryptedMsgKey);
+      Enigmail.hdrView.lastEncryptedMsgKey = null;
     }
   },
 
@@ -707,12 +705,28 @@ Enigmail.hdrView = {
   {
     EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: this.enigOnShowAttachmentContextMenu\n");
     // first, call the original function ...
-    onShowAttachmentContextMenu();
+
+    try {
+      // Thunderbird
+      onShowAttachmentItemContextMenu();
+    }
+    catch (ex) {
+      // SeaMonkey
+      onShowAttachmentContextMenu();
+    }
 
     // then, do our own additional stuff ...
-    var contextMenu = document.getElementById('attachmentListContext');
+
+    // Thunderbird
+    var contextMenu = document.getElementById('attachmentItemContext');
     var selectedAttachments = contextMenu.attachments;
 
+    if (! contextMenu) {
+      // SeaMonkey
+      contextMenu = document.getElementById('attachmentListContext');
+      selectedAttachments = attachmentList.selectedItems;
+    }
+
     var decryptOpenMenu = document.getElementById('enigmail_ctxDecryptOpen');
     var decryptSaveMenu = document.getElementById('enigmail_ctxDecryptSave');
     var importMenu = document.getElementById('enigmail_ctxImportKey');
@@ -890,8 +904,9 @@ if (messageHeaderSink) {
         throw Components.results.NS_NOINTERFACE;
       },
 
-      updateSecurityStatus: function (uriSpec, exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation)
+      updateSecurityStatus: function (uriSpec, exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation, uri)
       {
+        // uri is not used here; added for compatibility to other addons
         EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.updateSecurityStatus: uriSpec="+uriSpec+"\n");
 
         var msgUriSpec = Enigmail.msg.getCurrentMsgUriSpec();
